@@ -21,21 +21,13 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #
 
-import commands
 import os
 import shutil
-import subprocess
 import tempfile
-import urlparse
-
-try:
-    from debian.changelog import Version
-except ImportError:
-    # Prior to 0.1.15 the debian module was called debian_bundle
-    from debian_bundle.changelog import Version
 
 from bzrlib import (
     urlutils,
+    version_info as bzrlib_version,
     )
 from bzrlib.branch import Branch
 from bzrlib.bzrdir import BzrDir
@@ -61,32 +53,8 @@ from bzrlib.plugins.builddeb.config import (
     BUILD_TYPE_NATIVE,
     BUILD_TYPE_SPLIT,
     )
-from bzrlib.plugins.builddeb.errors import (
-    BuildFailedError,
-    DchError,
-    MissingChangelogError,
-    NoPreviousUpload,
-    PackageVersionNotPresent,
-    StrictBuildFailed,
-    )
-from bzrlib.plugins.builddeb.hooks import run_hook
 from bzrlib.plugins.builddeb.util import (
-    FORMAT_3_0_QUILT,
-    FORMAT_3_0_NATIVE,
-    component_from_orig_tarball,
     debuild_config,
-    dget_changes,
-    find_changelog,
-    find_last_distribution,
-    find_previous_upload,
-    get_source_format,
-    guess_build_type,
-    lookup_distribution,
-    md5sum_filename,
-    open_file,
-    open_file_via_transport,
-    tarball_name,
-    tree_contains_upstream_source,
     )
 
 dont_purge_opt = Option('dont-purge',
@@ -114,6 +82,50 @@ export_upstream_revision_opt = Option('export-upstream-revision',
     type=str)
 
 
+def _get_changelog_info(tree, last_version=None, package=None, distribution=None):
+    from bzrlib.plugins.builddeb.util import (
+        find_changelog,
+        find_last_distribution,
+        lookup_distribution,
+        )
+    from bzrlib.plugins.builddeb.errors import (
+        MissingChangelogError,
+        )
+    DEFAULT_FALLBACK_DISTRIBUTION = "debian"
+    current_version = last_version
+    try:
+        (changelog, top_level) = find_changelog(tree, False, max_blocks=2)
+    except MissingChangelogError:
+        top_level = False
+        changelog = None
+        if distribution is None:
+            distribution = DEFAULT_FALLBACK_DISTRIBUTION
+            note(gettext("No distribution specified, and no changelog, "
+                         "assuming '%s'"), distribution)
+    else:
+        if last_version is None:
+            current_version = changelog.version.upstream_version
+        if package is None:
+            package = changelog.package
+        if distribution is None:
+            distribution = find_last_distribution(changelog)
+            if distribution is not None:
+                note(gettext("Using distribution %s") % distribution)
+            else:
+                distribution = DEFAULT_FALLBACK_DISTRIBUTION
+                note(gettext("No distribution specified, and no previous "
+                             "distribution in changelog. Assuming '%s'"),
+                             distribution)
+    distribution = distribution.lower()
+    distribution_name = lookup_distribution(distribution)
+    if distribution_name is None:
+        raise BzrCommandError(gettext("Unknown target distribution: %s") \
+                    % distribution)
+    return (current_version, package, distribution, distribution_name,
+            changelog, top_level)
+
+
+
 class cmd_builddeb(Command):
     """Builds a Debian package from a branch.
 
@@ -189,6 +201,7 @@ class cmd_builddeb(Command):
         result_compat_opt, package_merge_opt]
 
     def _get_tree_and_branch(self, location):
+        import urlparse
         if location is None:
             location = "."
         is_local = urlparse.urlsplit(location)[0] in ('', 'file')
@@ -298,12 +311,18 @@ class cmd_builddeb(Command):
             quick=False, reuse=False, native=None,
             source=False, revision=None, result=None, package_merge=None,
             strict=False):
+        import commands
         from bzrlib.plugins.builddeb.source_distiller import (
                 FullSourceDistiller,
                 MergeModeDistiller,
                 NativeSourceDistiller,
                 )
         from bzrlib.plugins.builddeb.builder import DebBuild
+        from bzrlib.plugins.builddeb.errors import (
+            NoPreviousUpload,
+            StrictBuildFailed
+            )
+        from bzrlib.plugins.builddeb.hooks import run_hook
         from bzrlib.plugins.builddeb.upstream.branch import (
             LazyUpstreamBranchSource,
             )
@@ -317,6 +336,13 @@ class cmd_builddeb(Command):
         from bzrlib.plugins.builddeb.upstream.pristinetar import (
             PristineTarSource,
             )
+        from bzrlib.plugins.builddeb.util import (
+            dget_changes,
+            find_changelog,
+            find_previous_upload,
+            guess_build_type,
+            tree_contains_upstream_source,
+            )
 
         if result is not None:
             warning(gettext("--result is deprecated, use --result-dir instead"))
@@ -476,6 +502,9 @@ class cmd_get_orig_source(Command):
         from bzrlib.plugins.builddeb.upstream.pristinetar import (
             PristineTarSource,
             )
+        from bzrlib.plugins.builddeb.util import (
+            find_changelog,
+            )
         tree = WorkingTree.open_containing(directory)[0]
         config = debuild_config(tree, tree)
 
@@ -582,6 +611,9 @@ class cmd_merge_upstream(Command):
             changelog):
         from bzrlib.plugins.builddeb.merge_upstream import (
             changelog_add_new_version)
+        from bzrlib.plugins.builddeb.errors import (
+            DchError,
+            )
         try:
             changelog_add_new_version(tree, version, distribution_name,
                 changelog, package)
@@ -598,6 +630,9 @@ class cmd_merge_upstream(Command):
             DistributionBranch,
             DistributionBranchSet,
             )
+        from bzrlib.plugins.builddeb.util import (
+            component_from_orig_tarball,
+            )
         db = DistributionBranch(tree.branch, tree.branch, tree=tree)
         dbs = DistributionBranchSet()
         dbs.add_branch(db)
@@ -611,6 +646,7 @@ class cmd_merge_upstream(Command):
 
     def _fetch_tarball(self, package, version, orig_dir, locations, v3):
         from bzrlib.plugins.builddeb.repack_tarball import repack_tarball
+        from bzrlib.plugins.builddeb.util import tarball_name
         ret = []
         format = None
         for location in locations:
@@ -641,49 +677,32 @@ class cmd_merge_upstream(Command):
         return self._fetch_tarball(package, version, orig_dir,
             locations, v3)
 
-    def _get_changelog_info(self, tree, last_version, package, distribution):
-        current_version = last_version
-        try:
-            (changelog, top_level) = find_changelog(tree, False, max_blocks=2)
-            if last_version is None:
-                current_version = changelog.version.upstream_version
-            if package is None:
-                package = changelog.package
-            if distribution is None:
-                distribution = find_last_distribution(changelog)
-                if distribution is not None:
-                    note(gettext("Using distribution %s") % distribution)
-        except MissingChangelogError:
-            top_level = False
-            changelog = None
-        if distribution is None:
-            note("No distribution specified, and no changelog, "
-                    "assuming 'debian'")
-            distribution = "debian"
-        if package is None:
-            raise BzrCommandError("You did not specify --package, and "
-                    "there is no changelog from which to determine the "
-                    "package name, which is needed to know the name to "
-                    "give the .orig.tar.gz. Please specify --package.")
-        distribution = distribution.lower()
-        distribution_name = lookup_distribution(distribution)
-        if distribution_name is None:
-            raise BzrCommandError(gettext("Unknown target distribution: %s") \
-                        % distribution)
-        return (current_version, package, distribution, distribution_name,
-                changelog, top_level)
-
     def run(self, location=None, upstream_branch=None, version=None,
             distribution=None, package=None,
             directory=".", revision=None, merge_type=None,
             last_version=None, force=None, snapshot=False, launchpad=False):
+        try:
+            from debian.changelog import Version
+        except ImportError:
+            # Prior to 0.1.15 the debian module was called debian_bundle
+            from debian_bundle.changelog import Version
+
+        from bzrlib.plugins.builddeb.errors import PackageVersionNotPresent
+        from bzrlib.plugins.builddeb.hooks import run_hook
         from bzrlib.plugins.builddeb.upstream import (
-                TarfileSource,
-                UScanSource,
-                )
+            TarfileSource,
+            UScanSource,
+            )
         from bzrlib.plugins.builddeb.upstream.branch import (
-                UpstreamBranchSource,
-                )
+            UpstreamBranchSource,
+            )
+        from bzrlib.plugins.builddeb.util import (
+            FORMAT_3_0_QUILT,
+            FORMAT_3_0_NATIVE,
+            tree_get_source_format,
+            guess_build_type,
+            tree_contains_upstream_source,
+            )
 
         tree, _ = WorkingTree.open_containing(directory)
         tree.lock_write()
@@ -695,8 +714,14 @@ class cmd_merge_upstream(Command):
                         "command.")
             config = debuild_config(tree, tree)
             (current_version, package, distribution, distribution_name,
-             changelog, top_level) = self._get_changelog_info(tree, last_version,
+             changelog, top_level) = _get_changelog_info(tree, last_version,
                  package, distribution)
+            if package is None:
+                raise BzrCommandError("You did not specify --package, and "
+                        "there is no changelog from which to determine the "
+                        "package name, which is needed to know the name to "
+                        "give the .orig.tar.gz. Please specify --package.")
+
             contains_upstream_source = tree_contains_upstream_source(tree)
             if changelog is None:
                 changelog_version = None
@@ -807,7 +832,7 @@ class cmd_merge_upstream(Command):
                             revisions=upstream_revisions)
                     else:
                         raise
-                source_format = get_source_format(tree)
+                source_format = tree_get_source_format(tree)
                 v3 = (source_format in [
                     FORMAT_3_0_QUILT, FORMAT_3_0_NATIVE])
                 tarball_filenames = self._get_tarballs(config, tree, package,
@@ -873,9 +898,12 @@ class cmd_import_dsc(Command):
 
     def import_many(self, db, files_list, orig_target):
         from bzrlib.plugins.builddeb.import_dsc import (
-                DscCache,
-                DscComp,
-                )
+            DscCache,
+            DscComp,
+            )
+        from bzrlib.plugins.builddeb.util import (
+            open_file_via_transport,
+            )
         cache = DscCache()
         files_list.sort(cmp=DscComp(cache).cmp)
         if not os.path.exists(orig_target):
@@ -899,10 +927,17 @@ class cmd_import_dsc(Command):
             db.import_package(os.path.join(orig_target, filename))
 
     def run(self, files_list, file=None):
+        from bzrlib.plugins.builddeb.errors import (
+            MissingChangelogError,
+            )
         from bzrlib.plugins.builddeb.import_dsc import (
             DistributionBranch,
             DistributionBranchSet,
             )
+        from bzrlib.plugins.builddeb.util import (
+            find_changelog,
+            open_file,
+            )
         try:
             tree = WorkingTree.open_containing('.')[0]
         except NotBranchError:
@@ -1009,10 +1044,18 @@ class cmd_import_upstream(Command):
     takes_args = ['version', 'location', 'upstream_branch?']
 
     def run(self, version, location, upstream_branch=None, revision=None):
+        try:
+            from debian.changelog import Version
+        except ImportError:
+            # Prior to 0.1.15 the debian module was called debian_bundle
+            from debian_bundle.changelog import Version
         from bzrlib.plugins.builddeb.import_dsc import (
             DistributionBranch,
             DistributionBranchSet,
             )
+        from bzrlib.plugins.builddeb.util import (
+            md5sum_filename,
+            )
         # TODO: search for similarity etc.
         version = version.encode('utf8')
         branch, _ = Branch.open_containing('.')
@@ -1105,6 +1148,10 @@ class cmd_builddeb_do(Command):
     aliases = ['bd-do']
 
     def run(self, command_list=None):
+        import subprocess
+        from bzrlib.plugins.builddeb.errors import (
+            BuildFailedError,
+            )
         from bzrlib.plugins.builddeb.source_distiller import (
             MergeModeDistiller,
             )
@@ -1120,9 +1167,28 @@ class cmd_builddeb_do(Command):
         from bzrlib.plugins.builddeb.upstream.pristinetar import (
             PristineTarSource,
             )
+        from bzrlib.plugins.builddeb.hooks import run_hook
+        from bzrlib.plugins.builddeb.util import (
+            find_changelog,
+            guess_build_type,
+            tree_contains_upstream_source,
+            )
         t = WorkingTree.open_containing('.')[0]
+        self.add_cleanup(t.lock_read().unlock)
         config = debuild_config(t, t)
-        if config.build_type != BUILD_TYPE_MERGE:
+        (changelog, top_level) = find_changelog(t, False, max_blocks=2)
+
+        contains_upstream_source = tree_contains_upstream_source(t)
+        if changelog is None:
+            changelog_version = None
+        else:
+            changelog_version = changelog.version
+        build_type = config.build_type
+        if build_type is None:
+            build_type = guess_build_type(t, changelog_version,
+                contains_upstream_source)
+
+        if build_type != BUILD_TYPE_MERGE:
             raise BzrCommandError(gettext("This command only works for merge "
                 "mode packages. See /usr/share/doc/bzr-builddeb"
                 "/user_manual/merge.html for more information."))
@@ -1134,7 +1200,6 @@ class cmd_builddeb_do(Command):
             except KeyError:
                 command_list = ["/bin/sh"]
             give_instruction = True
-        (changelog, top_level) = find_changelog(t, True)
         build_dir = config.build_dir
         if build_dir is None:
             build_dir = default_build_dir
@@ -1207,6 +1272,9 @@ class cmd_mark_uploaded(Command):
             DistributionBranch,
             DistributionBranchSet,
             )
+        from bzrlib.plugins.builddeb.util import (
+            find_changelog,
+            )
         t = WorkingTree.open_containing('.')[0]
         t.lock_write()
         try:
@@ -1247,13 +1315,25 @@ class cmd_merge_package(Command):
     so that the user only needs to deal with packaging branch merge issues.
 
     In the opposite case a normal merge will be performed.
+
+    As of bzr 2.5 and bzr-builddeb 2.8.1, this functionality is automatically
+    provided as part of bzr merge.
     """
     takes_options = ['revision']
     takes_args = ['source']
 
+    if bzrlib_version >= (2, 5):
+        hidden = True
+
     def run(self, source, revision=None):
+        from bzrlib import ui
+        from bzrlib.merge import Merger
         from bzrlib.tag import _merge_tags_if_possible
         from bzrlib.plugins.builddeb.merge_package import fix_ancestry_as_needed
+        if 'pre_merge' in Merger.hooks:
+            ui.ui_factory.show_warning(
+                "The merge-package command is deprecated. Use 'bzr merge' "
+                "instead.")
         source_branch = None
         # Get the target branch.
         try:
