@@ -144,20 +144,25 @@ def find_changelog(t, merge, max_blocks=1):
         located at 'changelog' (rather than 'debian/changelog') if
         merge was given, False otherwise.
     """
-    changelog_file = 'debian/changelog'
     top_level = False
     t.lock_read()
     try:
+        changelog_file = 'debian/changelog'
         if not t.has_filename(changelog_file):
+            checked_files = ['debian/changelog']
             if merge:
                 # Assume LarstiQ's layout (.bzr in debian/)
                 changelog_file = 'changelog'
                 top_level = True
                 if not t.has_filename(changelog_file):
-                    raise MissingChangelogError('"debian/changelog" or '
-                            '"changelog"')
+                    checked_files.append(changelog_file)
+                    changelog_file = None
             else:
-                raise MissingChangelogError('"debian/changelog"')
+                changelog_file = None
+            if changelog_file is None:
+                if getattr(t, "abspath", None):
+                    checked_files = [t.abspath(f) for f in checked_files]
+                raise MissingChangelogError(" or ".join(checked_files))
         elif merge and t.has_filename('changelog'):
             # If it is a "top_level" package and debian is a symlink to
             # "." then it will have found debian/changelog. Try and detect
@@ -648,7 +653,7 @@ def tree_contains_upstream_source(tree):
     return (len(present_files - packaging_files) > 0)
 
 
-def get_source_format(tree):
+def tree_get_source_format(tree):
     """Retrieve the source format name from a package.
 
     :param path: Path to the package
@@ -657,7 +662,8 @@ def get_source_format(tree):
     filename = "debian/source/format"
     if not tree.has_filename(filename):
         return FORMAT_1_0
-    text = tree.get_file_text(tree.path2id(filename), filename)
+    file_id = tree.path2id(filename)
+    text = tree.get_file_text(file_id, filename)
     return text.strip()
 
 
@@ -677,7 +683,7 @@ def guess_build_type(tree, version, contains_upstream_source):
     :param contains_upstream_source: Whether this branch contains the upstream source.
     :return: A build_type value.
     """
-    source_format = get_source_format(tree)
+    source_format = tree_get_source_format(tree)
     if source_format in NATIVE_SOURCE_FORMATS:
         format_native = True
     elif source_format in NORMAL_SOURCE_FORMATS:
